//
//  SCAdSequencer.h
//  SCTVOSSDK
//
//  Copyright © 2019 smartclip. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <AVKit/AVKit.h>
#import "SCAdSlotController.h"
#import "SCAdFacadeDelegate.h"

NS_ASSUME_NONNULL_BEGIN

@class SCAdEnvironment, SCAdSlot, SCAdEvent, SCAdSessionController;

/**
 * Protocol that you implement in your view controller to
 * get informed about SCAdSequencer events
 */
@protocol SCAdSequencerDelegate <NSObject>
/**
 * The Sequencer calls this function when the
 * contentVideo should be played
 */
- (void)playContentVideoWith:(NSString*)urlString;

/// The sequencer tells you to pause the content video
- (void)pauseContentVideo;

/// The sequencer tells you to resume the content video
- (void)resumeContentVideo;

/**
 * Called when the content video was scrubbed
 */
- (NSArray<SCAdSlot*>*)userDidScrub:(NSArray<SCAdSlot*>*)removedAdSlots
            currentRelativePosition:(CGFloat)currentRelativePosition;

/**
 * Called when the content video has finished
 */
- (void)contentVideoFinished;

/**
 * Called when the content video fails for any reason
 */
- (void)contentVideoError:(NSError*)error;

/**
 * Called on for every ScAdInfo type change
 * - Parameter adInfo: current ScAdInfo
 */
- (void)onEventCallbackWithEvent:(SCAdEvent *)event;

/**
 * Called when sequencer has finished its playback sequence
 */
- (void)sequencerFinished;

/**
 * Called when the click through alert should be displayed
 */
- (void)displayClickThroughAlert:(UIAlertController*)alert;

/**
 * Called when the AVPlayer actually starts video playback
 */
- (void)adStartsPlayback;

/**
* Called when the presentationSize of the AVItem changed
*/
- (void)presentationSizeChangedTo:(CGSize)newSize;

- (nullable SCElementSize *)getPlayerSize;

- (nullable SCElementSize *)getViewportSize;

@end

/**
 * Use this class for instream use cases. Here you can specify each
 * AdSlot, it´s timing and are able to reinsert over-scrubbed adSlots
 */
@interface SCAdSequencer : NSObject
/// The configuration object for the complete adSlot session
@property (nonatomic, strong) SCAdConfiguration        *configuration;
/// Set your view controller as delegate of the sequener to get informed about events etc.
@property (nonatomic, weak) id<SCAdSequencerDelegate>   delegate;
/// Set your player implementation as facade delegate (so the sequnecer can talk to it)
@property (nonatomic, weak) id<SCAdFacadeDelegate>      facadeDelegate;
/// The array of your adSlots
@property (nonatomic, strong) NSMutableArray            *adSlots;
/// The url of your content video
@property (nonatomic, copy)   NSString                  *contentURL;

/// Default initializer
- (instancetype)initWithPlayerDelegate:(nonnull id<SCAdFacadeDelegate>)facadeDelegate sessionController:(SCAdSessionController*)sessionController;
/// Start the sequencer with this function
- (void)startSequencer;

/// Stop the sequencer with this function
- (void)stopSequencer;

/// Call this function to pause the sequencer
- (void)pauseSequencer;

/// Call this function to resume the sequencer (if paused)
- (void)resumeSequencer;

/// Call this function to to find out if the sequencer is paused
- (BOOL)isPaused;

/// Call this function to cleanup the sequencer
- (void)cleanup;

///Call this function when the user has pushed the skip button
- (void)skipAd;

///Call this function for a more detailed adInfo
- (void)getAdInfo:(SCAdInfoCompletionBlock)completion;

/// Get your publicAdSlot object here
- (void)getPublicAdSlot:(SCPublicAdSlotCompletionBlock)completion;

///Call this function when the user taps on the ad
- (void)onClickThrough;

///Add your playerView here for visibility checking
- (void)addViewForVisibilityCheck:(UIView*)view;

///Register any view, that covers your adView but belongs to your ad playback views (like soundButton, replayButton etc.)
- (void)registerFriendlyViews:(NSMutableArray*)friendlyViews;

///Call this function for a detailed adError object
- (void)getAdError:(SCAdErrorCompletionBlock)completion;

///Call this function when the user presses the mute button (on or off)
- (void)onMuteButtonPressed;
@end

NS_ASSUME_NONNULL_END
